==> kylin-ondevice-nlp-engine_1.0.0.0.orig.tar.gz: kylin-ondevice-nlp-engine-1.0.0.0/nlpserver/nlpserver.h <==
/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <kylin-ai/ai-engine/large-model/error.h>

#include <functional>
#include <memory>
#include <mutex>
#include <string>
#include <thread>
#include <unordered_map>

class server_context;
class server_task_result;
class gpt_params;

using CompletionResultCallback = std::function<void(const std::string &)>;

class NlpServer {
public:
    static NlpServer &getInstance();
    ~NlpServer();

    bool initialize();
    /**
     * @brief 初始化会话，返回会话ID
     * @param sessionId 会话ID
     */
    ai_engine::ErrorCode initSession(uint &sessionId);
    ai_engine::ErrorCode destroySession(uint sessionId);
    /**
     * @brief 设置文本扩写功能的回调函数
     *        调用此函数之前必须调用 initSession 函数初始化会话
     * @param callback  用于返回扩写结果的回调函数
     * @param sessionId 会话ID
     */
    void setCompletionResultCallback(CompletionResultCallback callback,
                                     uint sessionId);
    ai_engine::ErrorCode completion(const std::string &json_input,
                                    const uint sessionId, uint &task_id,
                                    int slot_id = -1);
    void cancelCompletion(const uint task_id);

private:
    NlpServer();
    void run();
    void terminate();
    int getNewTaskID();
    void checkConfigFile();
    void loadParamFromConfig();
    void addWaitingTaskID(int task_id);
    void removeWaitingTaskID(int task_id);
    ai_engine::ErrorCode handleStreamResults(int task_id, int sessionId,
                                             CompletionResultCallback callback);
    ai_engine::ErrorCode
    handleNonStreamResults(int task_id, int sessionId,
                           CompletionResultCallback callback);
    server_task_result receiveResult(int task_id);

private:
    bool isRunning_{false};
    std::thread taskThread_;
    std::mutex sessionMutex_;
    server_context *ctxServer_ = nullptr;
    std::unique_ptr<gpt_params> params_;
    std::string chatTemplate_;
    std::unordered_map<int, CompletionResultCallback> completionCallbacks_;
};

==> kylin-ondevice-nlp-engine_1.1.0.0.orig.tar.gz: kylin-ondevice-nlp-engine-1.1.0.0/nlpserver/nlpserver.h <==
/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <kylin-ai/ai-engine/large-model/aiengineerror.h>

#include <functional>
#include <memory>
#include <mutex>
#include <string>
#include <thread>
#include <unordered_map>

class server_context;
class server_task_result;
class gpt_params;

using CompletionResultCallback = std::function<void(const std::string &)>;

class NlpServer {
public:
    static NlpServer &getInstance();
    ~NlpServer();

    bool initialize();
    /**
     * @brief 初始化会话，返回会话ID
     * @param sessionId 会话ID
     */
    bool initSession(uint &sessionId, ai_engine::lm::EngineError &error);
    bool destroySession(uint sessionId);
    /**
     * @brief 设置文本扩写功能的回调函数
     *        调用此函数之前必须调用 initSession 函数初始化会话
     * @param callback  用于返回扩写结果的回调函数
     * @param sessionId 会话ID
     */
    void setCompletionResultCallback(CompletionResultCallback callback,
                                     uint sessionId);
    bool completion(const std::string &json_input, const uint sessionId,
                    uint &task_id, int slot_id,
                    ai_engine::lm::EngineError &error);
    void cancelCompletion(const uint task_id);

private:
    NlpServer();
    void run();
    void terminate();
    int getNewTaskID();
    void checkConfigFile();
    void loadParamFromConfig();
    void addWaitingTaskID(int task_id);
    void removeWaitingTaskID(int task_id);
    bool handleStreamResults(int task_id, int sessionId,
                             CompletionResultCallback callback,
                             ai_engine::lm::EngineError &error);
    bool handleNonStreamResults(int task_id, int sessionId,
                                CompletionResultCallback callback,
                                ai_engine::lm::EngineError &error);
    server_task_result receiveResult(int task_id);

private:
    bool isRunning_{false};
    std::thread taskThread_;
    std::mutex sessionMutex_;
    server_context *ctxServer_ = nullptr;
    std::unique_ptr<gpt_params> params_;
    std::string chatTemplate_;
    std::unordered_map<int, CompletionResultCallback> completionCallbacks_;
};
